#include <mpi.h>
#include "vtu_sam.h"

#include <vtkUnstructuredGrid.h>
#include <vtkXMLUnstructuredGridReader.h>
#include <vtkXMLPUnstructuredGridWriter.h>

#include <stdio.h>

int main(int argc, char **argv){
  MPI::Init(argc,argv);

  int rank = MPI::COMM_WORLD.Get_rank();
  int NProcs = MPI::COMM_WORLD.Get_size();

  char filename[1024];
  sprintf(filename, "coarse_slab_with_data/coarse_slab_with_data_%d.vtu", rank);

  vtkXMLUnstructuredGridReader *reader=vtkXMLUnstructuredGridReader::New();
  reader->SetFileName(filename);
  reader->Update();
  vtkUnstructuredGrid *ug = reader->GetOutput();

  int NPrivateNodes[] = {333, 333, 333, 332};

  // Data import.
  sam_init(ug, NPrivateNodes[rank]);

  // Load-balancing and data migration.
  sam_migrate();
  ug = sam_export_mesh(NPrivateNodes[rank]);

  vtkXMLPUnstructuredGridWriter *writer=vtkXMLPUnstructuredGridWriter::New();
  writer->SetFileName("load_balanced.pvtu");
  writer->SetDataModeToBinary();
  writer->SetNumberOfPieces(NProcs);
  writer->SetGhostLevel(1);
  writer->SetStartPiece(rank);
  writer->SetEndPiece(rank);
  writer->SetInput(ug);

  vtkZLibDataCompressor* compressor = vtkZLibDataCompressor::New();
  writer->SetCompressor(compressor);
  
  writer->Write();
  writer->Delete();
  compressor->Delete();

  // Cleanup
  sam_cleanup();
  
  MPI::Finalize();
  return(0);
}
